
# q-expansion of newform 630.4.k.d, downloaded from the LMFDB on 05 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 630
weight = 4
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[281, [1, 0]], [127, [1, 0]], [451, [0, -1]]]
aps_data = [[0, -2], [0, 0], [0, 5], [14, -21], [-32, 32], [15, 0], [-70, 70], [0, -15], [0, -42], [-90, 0], [85, -85], [0, -113], [-164, 0], [169, 0], [0, 326], [-44, 44], [-782, 782], [0, -658], [-1071, 1071], [-344, 0], [-431, 431], [0, -397], [-680, 0], [0, 1534], [-1234, 0], [0, 0], [0, 1691], [0, 162], [-1221, 1221], [274, 0], [529, 0], [0, 326], [894, -894], [895, 0], [0, -840], [-920, 920], [-910, 910], [0, 2068], [-3504, 0], [0, -1948], [-3030, 3030], [3133, 0], [0, -4422], [-2825, 2825], [-3170, 0], [3064, -3064], [460, 0], [4156, 0], [-2740, 2740], [0, -4921], [0, -4038], [2930, 0], [-1834, 1834], [-1014, 0], [0, 2816], [6788, -6788], [6198, -6198], [0, -3984], [-5127, 5127], [6196, 0], [-5789, 5789], [2108, 0], [-1121, 0], [8028, -8028], [0, -5093], [0, -3746], [0, -7261], [-2945, 0], [-11060, 11060], [6298, 0], [10092, -10092], [0, -11122], [-5287, 5287], [0, -13025], [13869, 0], [0, -1764], [-6064, 6064], [0, -9373], [0, 9580], [10107, -10107], [-8934, 0], [3389, 0], [-17514, 17514], [13457, 0], [0, 9428], [0, 16888], [822, 0], [0, 10865], [9518, 0], [9227, 0], [0, -5530], [-4812, 4812], [-8315, 8315], [-10212, 0], [0, 11229], [3786, 0], [0, -12206], [13952, -13952], [0, 10181], [0, 14055], [-6144, 0], [-14342, 14342], [-9658, 9658], [0, -14592], [-15699, 15699], [9779, -9779], [9454, 0], [0, -7668], [2052, -2052], [-19857, 0], [0, -10615], [9642, -9642], [-1694, 0], [4399, -4399], [5336, 0], [-18562, 18562], [-5687, 0], [24712, -24712], [0, 20552], [4472, 0], [10505, -10505], [19285, 0], [0, -18114], [-13606, 13606], [0, 6863], [-14458, 0], [0, -8278], [0, -29718], [-24069, 0], [0, 1973], [37085, -37085], [-4368, 0], [0, 25813], [18034, 0], [0, 9882], [-21845, 0], [-2976, 2976], [-4708, 4708], [-37950, 0], [1062, -1062], [-28852, 0], [0, -20580], [-41176, 41176], [29158, 0], [33415, -33415], [11506, 0], [34889, 0], [34818, -34818], [0, -12204], [0, 38422], [0, 26230], [1768, 0], [19887, 0], [0, -6436], [32065, -32065], [14394, 0], [0, 15367], [0, 19936], [45953, 0], [48782, -48782], [0, 51704], [-5916, 0], [-44981, 0], [0, -1980], [132, -132], [-28798, 28798], [-38707, 38707], [-22673, 22673]]
